 /*
 *  file      : configuser.c
 *  project   : zoomc
 *  with      : Gtk-2
 *
 *  copyright : (C) 2010 - 2011 by Claude Bulin
 *
 *  ZOOMC - GTK+ implementation of the GNU shell command
 *  GNU General Public License
 *
 *  This file is part of ZOOMC.
 * 
 *  ZOOMC is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  at your option) any later version.
 * 
 *  ZOOMC is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with ZOOMC.  If not, see <http://www.gnu.org/licenses/>.
 */



#ifdef HAVE_CONFIG_H
	#include "../config.h"
#endif

#ifdef ENABLE_NLS
	#include <libintl.h>
	#define _(String) gettext (String)
#endif

#include <gtk/gtk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#ifdef ENABLE_STATIC_LIBRARY
	#include "../libzoomc/lib.h"
#endif

#include "global.h"
#include "configuser.h"



typedef struct {
	gchar	**Larrbuf;
} CONF;

CONF	Conf = { NULL };

VAR_CONFIG Config = {
	0,		// WinPos_X
	0,		// WinPos_Y
	800,		// WinWidth
	520,		// WinHeight
	NULL,		// NameImg
	NULL,		// PathLoadImg
	NULL,		// PathSaveImg;
	NULL		// PathPochette
};




// TRUE SI p_dest EST DANS p_src SINON FALSE
//
gboolean config_new_strcmp (gchar *p_src, gchar *p_dest)
{
	gint	len_aiguille = strlen (p_dest);
	gint	len_meule;
	gint	len;
	gchar	*PtrBeginSrc = NULL;
	gchar	*PtrEndSrc = NULL;
	gchar	*PtrSrc = NULL;
	gchar	*PtrDest = p_dest;
	
	for (PtrBeginSrc = p_src; *PtrBeginSrc == ' ' || *PtrBeginSrc == '\t'; PtrBeginSrc ++);
	for (PtrEndSrc = PtrBeginSrc; *PtrEndSrc != ' ' && *PtrEndSrc != '\t'; PtrEndSrc ++);
	len_meule = PtrEndSrc - PtrBeginSrc;
	
	if (len_meule != len_aiguille) return (FALSE);
	
	len = 0;
	for (PtrSrc = PtrBeginSrc; PtrSrc < PtrEndSrc; PtrSrc ++, PtrDest ++) {
		if (*PtrSrc != *PtrDest) return (FALSE);;
		len ++;
	}
	
	return (len_meule == len ? TRUE : FALSE);
	
}
// 
// 
void config_print_data (gchar *p_str)
{
	gint	len = strlen (p_str);
	g_print("%s", p_str);
	while (len < 33) {
		g_print(" ");
		len ++;
	}
	g_print("= ");
}
// RETOURNE LE NOM COMPLET DU FICHIER DE CONFIGURATION
// 
gchar *config_get_pathname (void)
{
	return ((gchar *)g_strdup_printf ("%s/%s", getenv ("HOME"), ".config/zoomc/zoomc.conf"));
}
// RETOURNE UNE DEFINITION DU FICHIER DE CONFIGURATION
// 
gchar *config_get_target (gchar *p_buf)
{
	gchar	*Str = NULL;
	gchar	*Ptr = NULL;
	
	if ((Ptr = strchr (p_buf, '=')) != NULL) {
		Ptr ++;
		while (*Ptr && (*Ptr != ' ' && *Ptr != '\t')) Ptr ++;
		Ptr ++;
		Str = g_strdup (Ptr);
		if ((Ptr = strchr (Str, '\r')) != NULL) *Ptr = '\0';
		if ((Ptr = strchr (Str, '\n')) != NULL) *Ptr = '\0';
	}
	return ((gchar *)Str);
}
// 
// 
gboolean config_get_bool (gchar *value)
{
	gchar *ptr = value;

	while (*ptr) {
		*ptr = toupper (*ptr);
		ptr++;
	}
	return (!strncmp (value, "TRUE", strlen ("TRUE")) ? TRUE : FALSE);
}
// LECTURE BOOLEAN
// 
gboolean config_read_value_bool( gchar *p_Search, gboolean *p_ValueBool )
{
	gchar	*Ptr = NULL;
	gint	Cpt;
	
	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;
		
		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				*p_ValueBool = config_get_bool( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				config_print_data( p_Search );
				g_print( "%s\n", *p_ValueBool ? "TRUE" : "FALSE" );
				*Conf.Larrbuf[ 0 ] = '\0';
				return( TRUE );	
			}
		}
	}
	return( FALSE );	
}
// LECTURE NUMERIQUE
// 
gboolean config_read_value_atoi( gchar *p_Search, gint *p_ValueAtoi, gint p_Min, gint p_Max )
{
	gchar	*Ptr = NULL;
	gint	Cpt;
	
	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;
		
		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				*p_ValueAtoi = atoi( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				if( p_Min > -1 && *p_ValueAtoi < p_Min ) *p_ValueAtoi = p_Min;
				if( p_Max > -1 && *p_ValueAtoi > p_Max ) *p_ValueAtoi = p_Max;
				config_print_data( p_Search );
				g_print( "%d\n", *p_ValueAtoi ? *p_ValueAtoi : 0 );
				*Conf.Larrbuf[ 0 ] = '\0';
				return( TRUE );
			}
		}
	}
	return( FALSE );	
}
// LECTURE PATHNAME
// 
gboolean config_read_value_path( gchar *p_Search, gchar **p_ValuePath, gchar *p_default )
{
	gchar	*Ptr = NULL;
	gint	Cpt;
	
	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;
		
		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ] ))) {
				if( TRUE == libutils_test_dir_exist( Ptr )) {
					*p_ValuePath = Ptr;
				} else {
					if( NULL == p_default )	*p_ValuePath = g_strdup( getenv( "HOME" ));
					else			*p_ValuePath = g_strdup( p_default );
					g_free( Ptr );
					Ptr = NULL;
				}
			}
			else {
				if( NULL == p_default )	*p_ValuePath = g_strdup( getenv( "HOME" ));
				else			*p_ValuePath = g_strdup( p_default );
			}
			config_print_data( p_Search );
			g_print( "%s\n", *p_ValuePath ? *p_ValuePath : "" );
			*Conf.Larrbuf[ 0 ] = '\0';
			return( TRUE );	
		}
	}
	return( FALSE );	
}
// 
// 
gboolean config_read_value_str( gchar *p_Search, gchar **p_ValueStr )
{
	gchar	*Ptr = NULL;
	gint	Cpt;
	
	for( Cpt = 0; Conf.Larrbuf[ Cpt ]; Cpt++ ) {
		
		// Si commentaire ou ligne vide alors ligne suivante
		if( *Conf.Larrbuf[ Cpt ] == '#' || *Conf.Larrbuf[ Cpt ] == '\0') continue;
		
		if( TRUE == config_new_strcmp( Conf.Larrbuf[ Cpt ], p_Search )) {
			if( NULL != ( Ptr = config_get_target( Conf.Larrbuf[ Cpt ]))) {
				*p_ValueStr = g_strdup( Ptr );
				g_free( Ptr );
				Ptr = NULL;
				config_print_data( p_Search );
				g_print( "%s\n", *p_ValueStr ? *p_ValueStr : "" );
				*Conf.Larrbuf[ 0 ] = '\0';
				return( TRUE );	
			}
		}
	}
	return( FALSE );	
}
// 
// 
void config_verif_path (gchar **p_ValuePath, gchar *p_default)
{
	if (NULL == *p_ValuePath) {
		if (NULL == p_default)	*p_ValuePath = g_strdup (getenv ("HOME"));
		else			*p_ValuePath = g_strdup (p_default);
	}
}
// 	
// 	
gboolean config_open_conf_file( gchar *p_PathNameConfig )
{
	size_t 		SizeFile;
	gchar		*Buffer = NULL;
	FILE		*fp = NULL;
	
	Conf.Larrbuf = NULL;
	if(( SizeFile = libutils_get_size_file( p_PathNameConfig )) > 0 ) {
		if( NULL != ( fp = fopen( p_PathNameConfig, "r" ))) {
			Buffer = (gchar *)g_malloc0( sizeof(gchar) * ( SizeFile +10 ));
			SizeFile = fread( Buffer, SizeFile, 1, fp );
			fclose( fp );
			Conf.Larrbuf = g_strsplit( Buffer, "\n", 0 );
			g_free( Buffer );
			Buffer = NULL;
			return( TRUE );
		}
	}
	return( FALSE );
}
// 	
// 	
void config_close_conf_file( void )
{
	g_strfreev( Conf.Larrbuf );
	Conf.Larrbuf = NULL;
}
// LECTURE DU FICHIER DE CONFIGURATION
// 
void config_read (void)
{
	gchar	*PathNameConfig = config_get_pathname ();	// HOME /.config/zoomc/zoomc.conf
	
	g_print ("\n------------------------------------------------------------------------\n" );
	g_print ("%s\n", PathNameConfig);
	g_print ("------------------------------------------------------------------------\n" );
	
	// IF NOT EXIST CONFIG-FILE
	if( FALSE == libutils_test_file_exist( PathNameConfig )) {
		// GET PATHNAME
		gchar	*StrDir = g_strdup( PathNameConfig );
		// GET PATH
		gchar	*Ptr = strrchr( StrDir, '/' );
		
		// NOW StrDir IS PATH
		Ptr ++;
		*Ptr = '\0';
		// CREATION DU DOSSIER
		g_mkdir_with_parents (StrDir, 0700);
		// FREE MEM
		g_free( StrDir );	StrDir = Ptr = NULL;
		// CREATE CONFIG-FILE
		config_save ();
	}
	
	if( TRUE == config_open_conf_file( PathNameConfig )) {
		
		// READ ATOI
		// 
		config_read_value_atoi( "WIN_POS_X",				&Config.WinPos_X,       	 0,	-1);
		config_read_value_atoi( "WIN_POS_Y",				&Config.WinPos_Y,       	 0,	-1);
		config_read_value_atoi( "WIN_WIDTH",				&Config.WinWidth,        	-1,	-1);
		config_read_value_atoi( "WIN_HEIGHT",				&Config.WinHeight,       	-1,	-1);
		
		// READ BOOL
		// 
		config_read_value_str( "NAME_IMG",				&Config.NameImg);
		
		// READ PATH
		// 
		config_read_value_path( "PATH_LOAD_IMG",			&Config.PathLoadImg,		NULL);
		config_read_value_path( "PATH_SAVE_IMG",			&Config.PathSaveImg,		NULL);
		// DONT'T GET
		// if (FALSE == config_read_value_path (buf, "PATH_POCHE",	&Config.PathPochette,		NULL));

		// READ STRING
		// 
		
		config_close_conf_file();
			
		config_verif_path (&Config.PathLoadImg,			NULL);
		config_verif_path (&Config.PathSaveImg,			NULL);
	}
	// DONT'T VERIF
	// config_verif_path (&Config.PathPochette,		NULL);
	g_free( PathNameConfig );
	PathNameConfig = NULL;
}
//  SAUVEGARDE DU FICHIER DE CONFIGURATION
// 
void config_save (void)
{
	gchar	*PathNameConfig = config_get_pathname ();	// HOME /.config/zoomc/zoomc.conf
	FILE	*fp;
	
	if (NULL != (fp = fopen (PathNameConfig, "w"))) {
		fprintf (fp,   "############################################################################\n");
		fprintf (fp,   "## Fichier de configuration pour : %s\n", PACKAGE_STRING);
		fprintf (fp,   "##--------------------------------------------------------------------------\n");
		fprintf (fp,   "## File of configuration to : %s\n", PACKAGE_STRING);
		fprintf (fp,   "############################################################################\n");
		
		fprintf (fp, "\n");
		fprintf (fp, "WIN_POS_X                           = %d\n", Config.WinPos_X);
		fprintf (fp, "WIN_POS_Y                           = %d\n", Config.WinPos_Y);
		fprintf (fp, "WIN_WIDTH                           = %d\n", Config.WinWidth);
		fprintf (fp, "WIN_HEIGHT                          = %d\n", Config.WinHeight);
		fprintf (fp, "NAME_IMG                            = %s\n", Config.NameImg != NULL ? Config.NameImg : "" );
		fprintf (fp, "PATH_LOAD_IMG                       = %s\n", Config.PathLoadImg != NULL ? Config.PathLoadImg : "" );
		fprintf (fp, "PATH_SAVE_IMG                       = %s\n", Config.PathSaveImg != NULL ? Config.PathSaveImg : "" );;
		// DONT'T SAVE
		// fprintf (fp, "PATH_POCHE                          = %s\n", Config.PathPochette != NULL ? Config.PathPochette : "" );
		
		fprintf (fp, "\n");
		fclose (fp);
	}
	g_free( PathNameConfig );
	PathNameConfig = NULL;
}
// LIBERATION MEMOIRE
// 
void config_remove (void)
{
	g_free (Config.NameImg);			Config.NameImg = NULL;
	g_free (Config.PathLoadImg);			Config.PathLoadImg = NULL;
	g_free (Config.PathSaveImg);			Config.PathSaveImg = NULL;
	// DONT'T REMOVE
	// g_free (Config.PathPochette);			Config.PathPochette = NULL;
}

